#ifdef __OS400__

#include <qtqiconv.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include "api/yajl_iconv.h"

yajl_iconv_t * yajl_iconv_init( int rCCSID, int lCCSID ) {

    QtqCode_T icFrom;
    QtqCode_T icTo;
    yajl_iconv_t *res;

    memset(&icFrom, 0, sizeof(icFrom));
    memset(&icTo, 0, sizeof(icTo));

    res = malloc(sizeof(yajl_iconv_t));

    memset(res, 0, sizeof(yajl_iconv_t));
    res->rCCSID = rCCSID;
    res->lCCSID = lCCSID;


    /* convert from remote to local */

    icFrom.CCSID = res->rCCSID;
    icTo.CCSID   = res->lCCSID;
    res->l = QtqIconvOpen( &icTo, &icFrom);
    if (res->l.return_value == -1) {
       /* FIXME: handle error */
       int tempErr = errno;
       free(res);
       return NULL;
    }



    /* convert from local to remote */

    icFrom.CCSID = res->lCCSID;
    icTo.CCSID   = res->rCCSID;
    res->r = QtqIconvOpen( &icTo, &icFrom);
    if (res->r.return_value == -1) {
       /* FIXME: handle error */
       int tempErr = errno;
       free(res);
       return NULL;
    }

    return res;
}


#define CHUNK_SIZE 65536

char * yajl_iconv_dyn( iconv_t ic, const char *inpstr, int inplen, int *outlen ) {

    size_t inpleft, outleft, outsize;
    const char *inppos;
    char *outbuf, *outpos;
    int len;
    int rc;

    inpleft = inplen;
    inppos  = inpstr;

    outsize = inplen;
    outbuf  = malloc(outsize+1);
    outpos  = outbuf;
    outleft = outsize;

    while (1) {
       rc = (int)iconv( ic,
                        &inppos,
                        &inpleft,
                        &outpos,
                        &outleft );

       if (rc >= 0) break;
       if (-1 == rc && errno != E2BIG) {
           int tempErr = errno;
           /* FIXME: handle error */
           free(outbuf);
           return NULL;
       }

       /* E2BIG means the buffer wasn't large enough.  Try increasing it
          by one chunk size... */

       outsize += CHUNK_SIZE;
       len = outpos - outbuf;
       outbuf = realloc(outbuf, outsize+1);
       outpos = outbuf + len;
       outleft = outsize - len;
    }

    len = outsize - outleft;
    outbuf[len] = '\0';

    *outlen = len;
    return outbuf;
}


char *yajl_iconv_toRemote( yajl_iconv_t *res, const char *data ) {
    int outlen;
    return yajl_iconv_dyn( res->r,
                           data,
                           strlen(data),
                           &outlen );
}


char *yajl_iconv_toLocal( yajl_iconv_t *res, const char *data ) {
    int outlen;
    return yajl_iconv_dyn( res->l,
                           data,
                           strlen(data),
                           &outlen );
}


void yajl_iconv_free( yajl_iconv_t *res ) {

    iconv_close(res->l);
    iconv_close(res->r);
    free(res);

}

void yajl_iconv_string_free( char *str ) {
   free(str);
}
#endif

